// EigenCD.cpp : Defines the entry point for the console application.

//				Tests the Eigen function

#include "stdafx.h"
#include "Algo358.h"


using namespace System;
using namespace System::Reflection;		// for Assembly. ...
using namespace std;

void Eigen(complex<double> a[][maxA], int n, complex<double> va[], complex<double> ve[][maxA], bool herm, bool *error);
void CSVD(complex<double> a[][maxA], int m, int n, int p, int nu, int nv,
		double s[], complex<double> u[][maxA], complex<double> v[][maxA]);
void TestEigen();
void TimeStamp();
void WriteMatrix(complex<double> [][maxA], int, int);
void WriteMatrixZ(complex<double> [][maxA], int, int);
void MatMulSq(int n, complex<double> a[][maxA], complex<double> b[][maxA], complex<double> p[][maxA]);
void Hermitian(complex<double> a[][maxA], int n);

int ti;		// test case number, 1 to 4
double time0;

// ------- initialize
void init(void)
{
	char name[]    = "EigenCD - Computes Eigenvalues and Eigenvectors of a square complex matrix";
	char author1[] = "Copyright (c) 1981-2007 by author: Harry J. Smith,";
	char author2[] = "19628 Via Monte Dr., Saratoga, CA 95070.  All rights reserved.";
	char usage1[]  = "Usage: EigenCD [1 to 4] [>Test.out]";
	String^ version;		// = "C++ Version x.x.xxxx.xxxxx"
	Version aVersion = Assembly::GetExecutingAssembly()->GetName()->Version->ToString();
	version = "C++ Console Version " + aVersion.ToString() + " ";
	printf("\n%s\n", name);		// show credits
	int len;
	len = version->LastIndexOf(" ");
	char sta[40];
	for (int i = 0; i < len; i++)
		sta[i] = (char)version[i];
	sta[len] = 0;
	printf("%s\n", sta);
	printf("%s\n", author1);
	printf("%s\n\n", author2);
	printf("%s\n\n", usage1);	// show usage
} // init

// The main entry point for the application EigenCD
//int _tmain(int argc, _TCHAR* argv[])
int main(int argc, char* argv[])
{
	init();
	char tc[5] = "1";
	if (argc > 1)							// if arguments
		strncpy_s(tc, 3, argv[1], 2);		// Copy string from user input line
	ti = atoi(tc);
	if (ti < 2 || ti > 4)
		ti = 1;
	printf("Running test case %i of 4\n\n", ti);
	time0 = Environment::TickCount / 1000.0;
	TimeStamp();
	printf("\nEigenCD:\n");
	TestEigen();
	printf("EigenCD:\n");
	printf("Normal end of execution\n\n");
	TimeStamp();
	return 0;
} // main

void TestEigen()		//test Eigen
{
	int i, j, n;
	bool herm;
	bool error;
	complex<double> a[maxA][maxA];
	complex<double> u[maxA][maxA];
	complex<double> v[maxA][maxA];
	complex<double> w[maxA][maxA];
	complex<double> va[maxA];
	complex<double> ve[maxA][maxA];
	complex<double> cZero = complex<double>(0.0, 0.0);
	a[0][0] = complex<double>( 0.4499, -0.1267);		// Set the matrix
	a[0][1] = complex<double>(-0.2361,  0.0775);
	a[0][2] = complex<double>( 0.5008, -0.7799);
	a[0][3] = complex<double>(-0.7702, -0.3143);
	a[0][4] = complex<double>(-0.5084,  0.5621);
	a[0][5] = complex<double>(-0.5577, -0.0913);
	a[0][6] = complex<double>(-0.7207, -0.9798);
	a[0][7] = complex<double>(-0.5367,  0.4677);
	a[1][0] = complex<double>(-0.8432, -0.3443);
	a[1][1] = complex<double>( 0.0186, -0.6332);
	a[1][2] = complex<double>( 0.3505,  0.0166);
	a[1][3] = complex<double>(-0.8892,  0.2657);
	a[1][4] = complex<double>(-0.5066,  0.6005);
	a[1][5] = complex<double>( 0.1421,  0.0154);
	a[1][6] = complex<double>(-0.2327, -0.2606);
	a[1][7] = complex<double>(-0.4535,  0.4458);
	a[2][0] = complex<double>( 0.5896,  0.2601);
	a[2][1] = complex<double>( 0.8928,  0.0103);
	a[2][2] = complex<double>( 0.4350, -0.2666);
	a[2][3] = complex<double>(-0.7799, -0.5512);
	a[2][4] = complex<double>(-0.1022, -0.4500);
	a[2][5] = complex<double>(-0.6975, -0.8781);
	a[2][6] = complex<double>( 0.3364, -0.8415);
	a[2][7] = complex<double>( 0.3890,  0.6854);
	a[3][0] = complex<double>( 0.3911,  0.3234);
	a[3][1] = complex<double>(-0.5605,  0.7638);
	a[3][2] = complex<double>(-0.2009,  0.2707);
	a[3][3] = complex<double>( 0.0314, -0.4336);
	a[3][4] = complex<double>(-0.1046,  0.3267);
	a[3][5] = complex<double>(-0.8820, -0.9251);
	a[3][6] = complex<double>( 0.4480, -0.0054);
	a[3][7] = complex<double>( 0.5823, -0.2339);
	a[4][0] = complex<double>(-0.1395, -0.1561);
	a[4][1] = complex<double>( 0.3064,  0.0263);
	a[4][2] = complex<double>(-0.0975,  0.9019);
	a[4][3] = complex<double>( 0.2491,  0.5787);
	a[4][4] = complex<double>( 0.4106, -0.8102);
	a[4][5] = complex<double>(-0.7131,  0.5675);
	a[4][6] = complex<double>( 0.0759,  0.1507);
	a[4][7] = complex<double>( 0.1801, -0.3816);
	a[5][0] = complex<double>(-0.5889, -0.0238);
	a[5][1] = complex<double>( 0.9861, -0.4593);
	a[5][2] = complex<double>(-0.5219,  0.8539);
	a[5][3] = complex<double>( 0.6382, -0.0232);
	a[5][4] = complex<double>( 0.8741,  0.6109);
	a[5][5] = complex<double>(-0.2208, -0.1206);
	a[5][6] = complex<double>(-0.0868,  0.7716);
	a[5][7] = complex<double>(-0.4976,  0.4122);
	a[6][0] = complex<double>(-0.3180,  0.8621);
	a[6][1] = complex<double>( 0.3905, -0.8872);
	a[6][2] = complex<double>( 0.8140, -0.2542);
	a[6][3] = complex<double>(-0.2290,  0.4111);
	a[6][4] = complex<double>(-0.9723, -0.4355);
	a[6][5] = complex<double>( 0.5195,  0.4954);
	a[6][6] = complex<double>(-0.8843,  0.4552);
	a[6][7] = complex<double>( 0.9156, -0.5878);
	a[7][0] = complex<double>( 0.1596, -0.2962);
	a[7][1] = complex<double>( 0.0272,  0.3327);
	a[7][2] = complex<double>(-0.5806, -0.1952);
	a[7][3] = complex<double>(-0.9280, -0.5908);
	a[7][4] = complex<double>( 0.6263,  0.0384);
	a[7][5] = complex<double>( 0.0210, -0.7325);
	a[7][6] = complex<double>(-0.9907, -0.9717);
	a[7][7] = complex<double>( 0.0878, -0.9056);
	if (ti == 1)
	{
		n = 5;
		herm = false;
	}
	if (ti == 2)
	{
		n = 5;
		herm = true;
		Hermitian(a, n);	// Set the matrix
	}
	if (ti == 3)
	{
		n = 8;
		herm = false;
	}
	if (ti == 4)
	{
		n = 8;
		Hermitian(a, n);	// Set the matrix
		herm = true;
	}
		printf("Call the Eigen function to compute the Eigenvalues and Eigenvectors of\n");
	if (herm)
		printf("an N by N Hermitian");
	else
		printf("an N by N complex");
	printf(" matrix A, N = %i.\n", n);
	printf("\nMatrix A:\n\n");
	WriteMatrix(a, n, n);
	int p = 0;
	int nu = n;
	int nv = n;

	// INITIAL TESTS
	void PolNorm(complex<double> a[], int *n);
	complex<double> pp[9];
	pp[0] = complex<double>( 0.0000,  0.0000);
	pp[1] = complex<double>(-0.2009,  0.2707);
	pp[2] = complex<double>( 0.4106, -0.8102);
	int ppn = 3;
	PolNorm(pp, &ppn);
	void Laguer(complex<double> a[], int n, int *its, complex<double> *x);
	pp[0] = complex<double>( 0.4499, -0.1267);
	pp[1] = complex<double>(-0.2009,  0.2707);
	pp[2] = complex<double>( 0.4106, -0.8102);
	int its;
	// answer 0.80826026, 1.0269444
	complex<double> root;
	bool polish = true;
	Laguer(pp, 3, &its, &root);
	void PolRoots(complex<double> p[], int n, bool polish, complex<double> r[]);
	complex<double> roots[2];
	PolRoots(pp, 3, polish, roots);
	void CharPoly(complex<double> a[][maxA], int n, complex<double> p[]);
	CharPoly(a, n, pp);
	// END OF INITIAL TESTS

	Eigen(a, n, va, ve, herm, &error);
	printf("Eigenvalues va:\n\n");
	for (i = 0; i < n; i++)
		printf("  (%.14f,%.14f)\n", real(va[i]), imag(va[i]));
	printf("\nEigenvectors ve:\n\n");
	WriteMatrix(ve, n, n);
	MatMulSq(n, a, ve, u);
	printf("A * ve:\n\n");
	WriteMatrix(u, n, n);
	for (i = 0; i < n; i++)
		for (j = 0; j < n; j++)
			w[i][j] = cZero;
	for (i = 0; i < n; i++)
		w[i][i] = va[i];
	MatMulSq(n, ve, w, v);
	complex<double> maxErr = cZero;
	for (i = 0; i < n; i++)
		for (j = 0; j < n; j++)
		{
			w[i][j] = u[i][j] - v[i][j];
			if (abs(real(w[i][j])) > real(maxErr))
				maxErr = complex<double>(abs(real(w[i][j])), imag(maxErr));
			if (abs(imag(w[i][j])) > imag(maxErr))
				maxErr = complex<double>(real(maxErr), abs(imag(w[i][j])));
		}
	printf("A * ve - ve * Diag(va), should be zero: Max error = (%.3e,%.3e)\n\n", real(maxErr), imag(maxErr));
	WriteMatrixZ(w, n, n);
} // TestEigen

void Hermitian(complex<double> a[][maxA], int n)
// make a an n by n Hermitian matrix. use upper triangle to make lower triangle
{
	for (int i = 0; i < n - 1; i++)
	{
		for (int j = i + 1; j < n; j++)
			a[j][i] = conj(a[i][j]);
	}
	for (int k = 0; k < n; k++)
		a[k][k] = complex<double>(real(a[k][k]), 0.0);

} // Hermitian

void MatMulSq(int n, complex<double> a[][maxA], complex<double> b[][maxA], complex<double> p[][maxA])
// p = a * b. a, b, p all n by n complex matrices
{
	complex<double> cZero = complex<double>(0.0, 0.0);
	for (int i = 0; i < n; i++)
		for (int j = 0; j < n; j++)
		{
			p[i][j] = cZero;
			for (int k = 0; k < n; k++)
			{
				p[i][j] += a[i][k] * b[k][j];
			}
		}
} // MatMulSq

void WriteMatrix(complex<double> a[][maxA], int m, int n)
// print an m by n complex matrix
{
	for (int i = 0; i < m; i++)
	{
		for (int j = 0; j < n; j++)
		{
			printf(" ");
			if (real(a[i][j]) >= 0.0)
				printf(" ");
			else
				printf("-");
			if (real(a[i][j]) == 0.0)
				printf("0.0000");
			else
				printf("%.4f", abs(real(a[i][j])));
			if (imag(a[i][j]) >= 0.0)
				printf("+");
			else
				printf("-");
			if (imag(a[i][j]) == 0.0)
				printf("0.0000");
			else
				printf("%.4f", abs(imag(a[i][j])));
		}
		printf("\n");
	}
	printf("\n");
} // WriteMatrix

void WriteMatrixZ(complex<double> a[][maxA], int m, int n)
// print an m by n complex matrix that is near zero
{
	for (int i = 0; i < m; i++)
	{
		for (int j = 0; j < n; j++)
		{
			printf(" ");
			if (real(a[i][j]) >= 0.0)
				printf(" ");
			else
				printf("-");
			if (real(a[i][j]) == 0.0)
				printf("0.0000");
			else
				printf("%.0e", abs(real(a[i][j])));
			if (imag(a[i][j]) >= 0.0)
				printf("+");
			else
				printf("-");
			if (imag(a[i][j]) == 0.0)
				printf("0.0000");
			else
				printf("%.0e", abs(imag(a[i][j])));
		}
		printf("\n");
	}
	printf("\n");
} // WriteMatrixZ

void TimeStamp()
// printout the current MM/dd/yyyy hh:mm:ss AM/PM  s.ss sec. as a timestamp
{
	double time1 = Environment::TickCount / 1000.0; // TickCount = number of milliseconds since Windows started.
	double timeD = time1 - time0;
	time0 = time1;
	DateTime dt = DateTime::Now;
	String^ st = dt.ToString("MM/dd/yyyy hh:mm:ss tt");
	st += "  " + timeD.ToString("0.00") + " sec.";
	int len;
	len = 5 + st->LastIndexOf(" sec.");
	char sta[79];
	for (int i = 0; i < len; i++)
	{
		sta[i] = (char)st[i];
	}
	sta[len] = 0;
	printf("%s  %.2f sec.\n", sta, timeD);
} // TimeStamp

/* end of file EigenCD.cpp ------------------------------------------ */
